;    Last change:  SM    3 Feb 2004    2:51 pm
; REV.LSP

; Reverse direction of chosen object
; Handles ARCS LINES POLYLINES and LWPOLYLINES
; Polylines can be closed or open

; Paul Frylink 23 Sept 1997

; Start Point of object shown.
; Error routine added             PF 16 Oct 1997

; Pdmode & pdsize restored after program execution   PF 3 March 99
; Display start with circle instead of point (radius = 1% of viewsize) - pcp 25jan02 (pdmode/pdsize regen fix)

;***********************************************************

; Error routine

;;LCA - COMMENT: *error* defun no longer evaluates as a list.
;;LCA - COMMENT: *error* defun no longer evaluates as a list.
(defun *error* (msg)
   (princ "\nerror: ")
   (princ msg)
   (finish)
)

(defun finish ()
   (command ".erase" ptss "") ; erase start point markers
   (setvar "cmdecho" 1)
;;   (setvar "pdmode" pd_mode)
;;   (setvar "pdsize" pd_size)
   (princ)
) ; finish

;******************

; This procedure processes polylines (LWPOLYLINES and POLYLINES)
; It Reverses the order of vertices and matches up the
; Correct bulge factors

(defun procpoly (pcode vert type)

   ; If group code 70 contains a 1 (ie, is odd) then polyline is closed

   (if (/= (/ pcode 2) (/ pcode 2.0))
      (setq vlist (append (list (car vlist)) (reverse vlist))) ; Polyline is closed
      (setq vlist (reverse vlist))                             ; Polyline is open
   ) ; if

   (setq count 0)

   (while (/= count vert) ; loop through all vertices

      (setq leg (nth count vlist)
           bulge (cdr (assoc '42 (nth (1+ count) vlist))) ; bulge factor
      ) ; setq

      (if (not bulge)
        (setq bulge 0)  ; last leg
         (setq bulge (* -1 bulge) ; reverse arc - if any
              leg (subst (cons '42 bulge) (assoc '42 leg) leg)
        ) ; setq
      ) ; if

      (if (eq "LWPOLY" type)
        (setq elist (append elist leg))
       (entmake leg)
      ) ; setq

      (setq count (1+ count))

   ) ; while

) ; procrev

;***********************************************************

; Process LWPOLYLINES (R14)

(defun revlwpoly (en ed / count vlist hlist n elist pcode)

   (setq count 0
        vlist nil  ; vertices
        hlist nil  ; header info
       end (cdr (assoc '10 ed))
   ) ; setq

   ; Assemble a list of all vertices.

   (while (setq n (nth count ed))
      (if (= 10 (car n)) ; start of segments
        ; assumes group codes are always 10,40,41,42
         (setq vlist (append vlist (list (list n (nth (+ 1 count) ed) (nth (+ 2 count) ed) (nth (+ 3 count) ed))))
             start (cdr n)
               count (+ 4 count)
        ) ; setq
        (setq hlist (append hlist (list n)) ; list of header data
              count (1+ count)
        ) ; setq
      ); if
   ) ; while

   (setq elist hlist)
   (procpoly (cdr (assoc '70 ed)) (cdr (assoc '90 ed)) "LWPOLY")
   (entmod elist)

   (drawpt start end)

) ; revlwpoly

;***************

; Process POLYLINES (R13 or R14)

(defun revpoly (en ed / vlist count delen)

   (entmake ed) ; start making complex entity (polyline)

   (setq delen en
         pcode (cdr (assoc '70 ed))
         en (entnext (car en))
         ed (entget en)
       end (cdr (assoc '10 ed))
       vlist nil
   ) ; setq

   ; Assemble list of vertices

   (while (/= "SEQEND" (cdr (assoc '0 ed)))
      (setq vlist (append vlist (list ed))
           start (cdr (assoc '10 ed))
           en (entnext en)
         ed (entget en)
     ) ; setq
   ) ; while

   (procpoly pcode (length vlist) "POLY")

   (command ".erase" delen "")
   (entmake ed) ; finish off object creation

   (drawpt start end)

) ; defun revpoly

;***************

; Reverse direction of a LINE

(defun revline (en ed / start end)

   (setq start (cdr (assoc '11 ed))
         end (cdr (assoc '10 ed))
       ed (subst (cons '10 start) (assoc '10 ed) ed)
       ed (subst (cons '11 end) (assoc '11 ed) ed)
   ) ; setq

   (entmod ed)
   (drawpt start end)

) ; revline

;****************

; Reverse Direction of an ARC

(defun revarc (en ed / start end)

   (initget 0 "Yes No")
   (setq yn (getkword "\nArc must be converted to a POLYLINE - Proceed Y/N? [Y]"))
   (if (or (not yn) (eq "Yes" yn))
      (progn
        (command ".pedit" en "Y" "X")
       (setq en (entlast)
             ed (entget en)
       ) ; setq
       (if (eq "LWPOLYLINE" (cdr (assoc 0 ed)))
          (revlwpoly en ed) ; R14
         (revpoly en ed)   ; R13
       ) ; if
     ) ; progn
   ) ; if

) ; revarc

;********************************************

(defun drawpt (start end)
   ; Show direction of object by drawing circle at start
   ; first check if point already drawn at other end

   (setq count 0)

   (while (< count (sslength ptss)) ; count through all points drawn

      (setq en (ssname ptss count)
            ed (entget en)
            pt (cdr (assoc '10 ed))
            count (1+ count)
      ) ; setq

      (if (= 0 (distance end pt))
         (progn ; yes
            (command ".erase" en "")
            (ssdel en ptss)
         ) ; progn if yes
      ) ; if

   ) ; while
   (setq vrad 1.0)
   (command ".circle" start (* (getvar "viewsize") 0.01))
   (setq ptss (ssadd (entlast) ptss)) ; add point to selection set

) ; defun drawpt

;***************
;***************

; Main Routine
; Ask user to select object and then call appropriate routine.

(defun c:rev_poly ()

   ; setup

;;   (setq pd_mode (getvar "pdmode")
;;         pd_size (getvar "pdsize")
;;   )
   (setvar "cmdecho" 0)
;;   (setvar "pdmode" 33)
;;   (setvar "pdsize" -5)
   (setq ptss (ssadd))
;   (alert "Start point of chosen object(s) will be shown by a circle.")

   ; Loop while user picks objects

   (while (setq en (entsel "\nSelect ARC, LINE, POLYLINE or LWPOLYLINE to reverse =>"))

      (setq ed (entget (car en))
          object (cdr (assoc 0 ed))
      ) ; setq

      (cond
         ((= "LWPOLYLINE" object) (revlwpoly en ed))
        ((= "POLYLINE" object) (revpoly en ed))
        ((= "LINE" object) (revline en ed))
        ((= "ARC" object) (revarc en ed))
        (t (princ "\nCan NOT reverse that object -> "))
      ) ; cond
      (princ object)

   ) ; while

   (finish)

) ; rev

(setvar "cmdecho" 0)
(princ "REV_POLY loaded - type REV_POLY to reverse an object")
(setvar "cmdecho" 1)
(princ)
